"""
    Sliver Implant Framework
    Copyright (C) 2021  Bishop Fox

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


import logging
from typing import AsyncGenerator, Dict, Iterable, List, Optional, Union

import grpc

from sliver.pb.commonpb.common_pb2 import Request

from .beacon import InteractiveBeacon
from .config import SliverClientConfig
from .pb.rpcpb.services_pb2_grpc import SliverRPCStub
from .protobuf import client_pb2, common_pb2, sliver_pb2
from .session import InteractiveSession

KB = 1024
MB = 1024 * KB
GB = 1024 * MB
TIMEOUT = 60


class BaseClient(object):

    # 2GB triggers an overflow error in the gRPC library so we do 2GB-1
    MAX_MESSAGE_LENGTH = (2 * GB) - 1

    KEEP_ALIVE_TIMEOUT = 10000
    CERT_COMMON_NAME = "multiplayer"

    def __init__(self, config: SliverClientConfig):
        self.config = config
        self._channel: grpc.aio.Channel = None  # type: ignore[assignment]
        self._stub: SliverRPCStub = None  # type: ignore[assignment]
        self._log = logging.getLogger(self.__class__.__name__)

    def is_connected(self) -> bool:
        return self._channel is not None

    @property
    def target(self) -> str:
        return "%s:%d" % (
            self.config.lhost,
            self.config.lport,
        )

    @property
    def credentials(self) -> grpc.ChannelCredentials:
        return grpc.composite_channel_credentials(
            grpc.ssl_channel_credentials(
                root_certificates=self.config.ca_certificate.encode(),
                private_key=self.config.private_key.encode(),
                certificate_chain=self.config.certificate.encode(),
            ),
            grpc.access_token_call_credentials(
                access_token=self.config.token,
            ),
        )

    @property
    def options(self):
        return [
            ("grpc.keepalive_timeout_ms", self.KEEP_ALIVE_TIMEOUT),
            ("grpc.ssl_target_name_override", self.CERT_COMMON_NAME),
            ("grpc.max_send_message_length", self.MAX_MESSAGE_LENGTH),
            ("grpc.max_receive_message_length", self.MAX_MESSAGE_LENGTH),
        ]


class SliverClient(BaseClient):

    """Asyncio client implementation"""

    beacon_event_types = ["beacon-registered"]
    session_event_types = ["session-connected", "session-disconnected"]
    job_event_types = ["job-started", "job-stopped"]
    canary_event_types = ["canary"]

    async def connect(self) -> client_pb2.Version:
        """Establish a connection to the Sliver server

        :return: Protobuf Version object, containing the server's version information
        :rtype: client_pb2.Version
        """
        self._channel = grpc.aio.secure_channel(
            target=self.target,
            credentials=self.credentials,
            options=self.options,
        )
        self._stub = SliverRPCStub(self._channel)
        return await self.version()

    async def interact_session(
        self, session_id: str, timeout=TIMEOUT
    ) -> Optional[InteractiveSession]:
        """Interact with a session, returns an :class:`InteractiveSession`

        :param session_id: Session ID
        :type session_id: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :return: An interactive session
        :rtype: Optional[InteractiveSession]
        """
        session = await self.session_by_id(session_id, timeout)
        if session:
            return InteractiveSession(session, self._channel, timeout)

    async def interact_beacon(
        self, beacon_id: str, timeout=TIMEOUT
    ) -> Optional[InteractiveBeacon]:
        """Interact with a beacon, returns an :class:`InteractiveBeacon`

        :param beacon_id: Beacon ID
        :type beacon_id: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :return: An interactive beacon
        :rtype: Optional[AsyncInteractiveBeacon]
        """
        beacon = await self.beacon_by_id(beacon_id, timeout)
        if beacon:
            return InteractiveBeacon(beacon, self._channel, timeout)

    async def session_by_id(
        self, session_id: str, timeout=TIMEOUT
    ) -> Optional[client_pb2.Session]:
        """Get the session information from a session ID

        :param session_id: Session ID
        :type session_id: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :return: Protobuf Session object
        :rtype: Optional[client_pb2.Session]
        """
        sessions = await self.sessions(timeout)
        for session in sessions:
            if session.ID == session_id:
                return session

    async def beacon_by_id(
        self, beacon_id: str, timeout=TIMEOUT
    ) -> Optional[client_pb2.Beacon]:
        """Get the beacon information from a beacon ID

        :param beacon_id: Beacon ID
        :type beacon_id: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :return: Protobuf Beacon object
        :rtype: Union[client_pb2.Beacon, None]
        """
        beacons = await self.beacons(timeout)
        for beacon in beacons:
            if beacon.ID == beacon_id:
                return beacon

    async def events(self) -> AsyncGenerator[client_pb2.Event, None]:
        """All events

        :yield: A stream of events
        :rtype: client_pb2.Event
        """
        async for event in self._stub.Events(common_pb2.Empty()):
            yield event

    async def on(
        self, event_types: Union[str, List[str]]
    ) -> AsyncGenerator[client_pb2.Event, None]:
        """Iterate on a specific event or list of events

        :param event_types: An event type or list of event types
        :type event_types: Union[str, List[str]]
        :yield: A stream of events of the given type(s)
        :rtype: client_pb2.Event
        """
        if isinstance(event_types, str):
            event_types = [event_types]
        async for event in self.events():
            if event.EventType in event_types:
                yield event

    async def version(self, timeout=TIMEOUT) -> client_pb2.Version:
        """Get server version information

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf Version object
        :rtype: client_pb2.Version
        """
        return await self._stub.GetVersion(common_pb2.Empty(), timeout=timeout)

    async def operators(self, timeout=TIMEOUT) -> List[client_pb2.Operator]:
        """Get a list of operators and their online status

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of protobuf Operator objects
        :rtype: List[client_pb2.Operator]
        """
        operators = await self._stub.GetOperators(common_pb2.Empty(), timeout=timeout)
        return list(operators.Operators)

    async def sessions(self, timeout=TIMEOUT) -> List[client_pb2.Session]:
        """Get a list of active sessions

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of protobuf Session objects
        :rtype: List[client_pb2.Session]
        """
        sessions: client_pb2.Sessions = await self._stub.GetSessions(
            common_pb2.Empty(), timeout=timeout
        )
        return list(sessions.Sessions)

    async def rename_session(self, session_id: str, name: str, timeout=TIMEOUT) -> None:
        """Rename a session

        :param session_id: Session ID to update
        :type session_id: str
        :param name: Rename session to this value
        :type name: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: None
        :rtype: None
        """
        rename_req = client_pb2.RenameReq(SessionID=session_id, Name=name)
        await self._stub.Rename(rename_req, timeout=timeout)

    async def kill_session(self, session_id: str, force=False, timeout=TIMEOUT) -> None:
        """Kill a session

        :param session_id: Session ID to kill
        :type session_id: str
        :param force: Force kill the session, defaults to False
        :type force: bool, optional
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        """
        request = Request(SessionID=session_id, Timeout=timeout)
        kill_req = sliver_pb2.KillReq(Force=force, Request=request)
        await self._stub.Kill(kill_req, timeout=timeout)

    async def beacons(self, timeout=TIMEOUT) -> List[client_pb2.Beacon]:
        """Get a list of active beacons

        :param timeout: gRPC timeout, defaults to 60 seconds
        :rtype: List[client_pb2.Beacon]
        """
        beacons: client_pb2.Beacons = await self._stub.GetBeacons(
            common_pb2.Empty(), timeout=timeout
        )
        return list(beacons.Beacons)

    async def rename_beacon(self, beacon_id: str, name: str, timeout=TIMEOUT) -> None:
        """Rename a beacon

        :param beacon_id: Beacon ID to update
        :type beacon_id: str
        :param name: Rename beacon to this value
        :type name: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: None
        :rtype: None
        """
        rename_req = client_pb2.RenameReq(BeaconID=beacon_id, Name=name)
        await self._stub.Rename(rename_req, timeout=timeout)

    async def kill_beacon(self, beacon_id: str, timeout=TIMEOUT) -> None:
        """Kill a beacon

        :param beacon_id: Numeric beacon ID to remove
        :type beacon_id: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        """
        beacon_rm = client_pb2.Beacon(ID=beacon_id)
        await self._stub.RmBeacon(beacon_rm, timeout=timeout)

    async def beacon_tasks(
        self, beacon_id: str, timeout=TIMEOUT
    ) -> List[client_pb2.BeaconTask]:
        """Get a list of tasks for a beacon

        :param beacon_id: Beacon ID to get tasks for
        :type beacon_id: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of protobuf Task objects
        :rtype: List[client_pb2.Task]
        """
        beacon = client_pb2.Beacon(ID=beacon_id)
        tasks = await self._stub.GetBeaconTasks(beacon, timeout=timeout)
        return list(tasks.Tasks)

    async def beacon_task_content(
        self, task_id: str, timeout=TIMEOUT
    ) -> List[client_pb2.BeaconTask]:
        """Get a list of tasks for a beacon

        :param task_id: Task ID get contents for
        :type task_id: sts
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of protobuf Task objects
        :rtype: List[client_pb2.Task]
        """
        beacon = client_pb2.Beacon(ID=task_id)
        task = await self._stub.GetBeaconTaskContent(beacon, timeout=timeout)
        return task

    async def jobs(self, timeout=TIMEOUT) -> List[client_pb2.Job]:
        """Get a list of active jobs

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of protobuf Job objects
        :rtype: List[client_pb2.Job]
        """
        jobs: client_pb2.Jobs = await self._stub.GetJobs(
            common_pb2.Empty(), timeout=timeout
        )
        return list(jobs.Active)

    async def job_by_id(self, job_id: int, timeout=TIMEOUT) -> Optional[client_pb2.Job]:
        """Get job by id

        :param job_id: Beacon ID to get tasks for
        :type job_id: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of protobuf Job objects
        :rtype: List[client_pb2.Job]
        """
        for job in await self.jobs(timeout=timeout):
            if job.ID == job_id:
                return job

    async def job_by_port(
        self, job_port: int, timeout=TIMEOUT
    ) -> Optional[client_pb2.Job]:
        """Get job by port

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of protobuf Job objects
        :rtype: List[client_pb2.Job]
        """
        for job in await self.jobs(timeout=timeout):
            if job.Port == job_port:
                return job

    async def kill_job(self, job_id: int, timeout=TIMEOUT) -> client_pb2.KillJob:
        """Kill a job

        :param job_id: Numeric job ID to kill
        :type job_id: int
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf KillJob object
        :rtype: client_pb2.KillJob
        """
        kill_req = client_pb2.KillJobReq(ID=job_id)
        return await self._stub.KillJob(kill_req, timeout=timeout)

    async def start_mtls_listener(
        self,
        host: str = "0.0.0.0",
        port: int = 8888,
        persistent: bool = False,
        timeout=TIMEOUT,
    ) -> client_pb2.MTLSListener:
        """Start a mutual TLS (mTLS) C2 listener

        :param host: Host interface to bind the listener to, an empty string will bind to all interfaces
        :type host: str
        :param port: TCP port number to start listener on
        :type port: int
        :param persistent: Register the listener as a persistent job (automatically start with server), defaults to False
        :type persistent: bool, optional
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf MTLSListener object
        :rtype: client_pb2.MTLSListener
        """
        mtls_req = client_pb2.MTLSListenerReq(
            Host=host, Port=port, Persistent=persistent
        )
        return await self._stub.StartMTLSListener(mtls_req, timeout=timeout)

    async def start_wg_listener(
        self,
        tun_ip: str,
        port: int = 53,
        n_port: int = 8888,
        key_port: int = 1337,
        persistent: bool = False,
        timeout: int = TIMEOUT,
    ) -> client_pb2.WGListener:
        """Start a WireGuard (wg) C2 listener

        :param tun_ip: Virtual TUN IP listen address
        :type tun_ip: str
        :param port: UDP port to start listener on
        :type port: int
        :param n_port: Virtual TUN port number
        :type n_port: int
        :param key_port: Virtual TUN port number for key exchanges
        :type key_port: int
        :param persistent: Register the listener as a persistent job (automatically start with server), defaults to False
        :type persistent: bool, optional
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf WGListener object
        :rtype: client_pb2.WGListener
        """
        wg_req = client_pb2.WGListenerReq(
            TunIP=tun_ip,
            Port=port,
            NPort=n_port,
            KeyPort=key_port,
            Persistent=persistent,
        )
        return await self._stub.StartWGListener(wg_req, timeout=timeout)

    async def start_dns_listener(
        self,
        domains: List[str],
        host: str = "0.0.0.0",
        port: int = 53,
        canaries: bool = True,
        persistent: bool = False,
        enforce_otp=True,
        timeout: int = TIMEOUT,
    ) -> client_pb2.DNSListener:
        """Start a DNS C2 listener

        :param domains: C2 domains to listen for
        :type domains: List[str]
        :param canaries: Enable/disable DNS canaries
        :type canaries: bool
        :param host: Host interface to bind the listener to, an empty string will bind to all interfaces
        :type host: str
        :param port: TCP port number to start listener on
        :type port: int
        :param persistent: Register the listener as a persistent job (automatically start with server), defaults to False
        :type persistent: bool, optional
        :param enforce_otp: Enforce OTP auth for DNS C2, defaults to True
        :type enforce_otp: bool, optional
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf DNSListener object
        :rtype: client_pb2.DNSListener
        """
        # Ensure domains always have a trailing dot
        domains = list(map(lambda d: d + "." if d[-1] != "." else d, domains))

        dns_req = client_pb2.DNSListenerReq(
            Domains=domains,
            Canaries=canaries,
            Host=host,
            Port=port,
            Persistent=persistent,
            EnforceOTP=enforce_otp,
        )
        return await self._stub.StartDNSListener(dns_req, timeout=timeout)

    async def start_http_listener(
        self,
        host: str = "0.0.0.0",
        port: int = 80,
        website: str = "",
        domain: str = "",
        persistent: bool = False,
        timeout: int = TIMEOUT,
    ) -> client_pb2.HTTPListener:
        """Start an HTTP C2 listener


        :param host: Host interface to bind the listener to, an empty string will bind to all interfaces
        :type host: str
        :param port: TCP port number to start listener on
        :type port: int
        :param website: Name of the "website" to host on listener
        :type website: str
        :param domain: Domain name for HTTP server (one domain per listener)
        :type domain: str
        :param persistent: Register the listener as a persistent job (automatically start with server), defaults to False
        :type persistent: bool, optional
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf HTTPListener object (NOTE: HTTP/HTTPS both return HTTPListener objects)
        :rtype: client_pb2.HTTPListener
        """
        http_req = client_pb2.HTTPListenerReq(
            Domain=domain,
            Host=host,
            Port=port,
            Secure=False,
            Website=website,
            Persistent=persistent,
        )
        return await self._stub.StartHTTPListener(http_req, timeout=timeout)

    async def start_https_listener(
        self,
        host: str = "0.0.0.0",
        port: int = 443,
        website: str = "",
        domain: str = "",
        cert: bytes = b"",
        key: bytes = b"",
        acme: bool = False,
        persistent: bool = False,
        enforce_otp: bool = True,
        randomize_jarm: bool = True,
        long_poll_timeout: int = 1,
        long_poll_jitter: int = 2,
        timeout: int = TIMEOUT,
    ) -> client_pb2.HTTPListener:
        """Start an HTTPS C2 listener

        :param domain: Domain name for HTTPS server (one domain per listener)
        :type domain: str
        :param host: Host interface to bind the listener to, an empty string will bind to all interfaces
        :type host: str
        :param port: TCP port number to start listener on
        :type port: int
        :param website: Name of the "website" to host on listener
        :type website: str
        :param cert: TLS certificate (leave blank to generate self-signed certificate)
        :type cert: bytes
        :param key: TLS private key (leave blank to generate self-signed certificate)
        :type key: bytes
        :param acme: Automatically provision TLS certificate using ACME (i.e., Let's Encrypt)
        :type acme: bool
        :param persistent: Register the listener as a persistent job (automatically start with server), defaults to False
        :type persistent: bool, optional
        :param enforce_otp: Enforce OTP auth for HTTPS C2, defaults to True
        :type enforce_otp: bool, optional
        :param randomize_jarm: Randomize JARM fingerprint for HTTPS C2, defaults to True
        :type randomize_jarm: bool, optional
        :param long_poll_timeout: Long poll timeout for HTTPS C2, defaults to 1
        :type long_poll_timeout: int, optional
        :param long_poll_jitter: Long poll jitter for HTTPS C2, defaults to 2
        :type long_poll_jitter: int, optional
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf HTTPListener object (NOTE: HTTP/HTTPS both return HTTPListener objects)
        :rtype: client_pb2.HTTPListener
        """
        https_req = client_pb2.HTTPListenerReq(
            Domain=domain,
            Host=host,
            Port=port,
            Secure=True,
            Website=website,
            Cert=cert,
            Key=key,
            ACME=acme,
            Persistent=persistent,
            EnforceOTP=enforce_otp,
            LongPollTimeout=long_poll_timeout,
            LongPollJitter=long_poll_jitter,
            RandomizeJARM=randomize_jarm,
        )
        return await self._stub.StartHTTPSListener(https_req, timeout=timeout)

    async def start_tcp_stager_listener(
        self, host: str, port: int, data: bytes, timeout=TIMEOUT
    ) -> client_pb2.StagerListener:
        """Start a TCP stager listener

        :param host: Host interface to bind the listener to, an empty string will bind to all interfaces
        :type host: str
        :param port: TCP port number to start listener on
        :type port: int
        :param data: Binary data of stage to host on listener
        :type data: bytes
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf StagerListener object
        :rtype: client_pb2.StagerListener
        """
        stage_req = client_pb2.StagerListenerReq(
            Protocol=client_pb2.TCP,
            Host=host,
            Port=port,
            Data=data,
        )
        return await self._stub.StartTCPStagerListener(stage_req, timeout=timeout)

    async def start_http_stager_listener(
        self,
        host: str,
        port: int,
        data: bytes,
        timeout: int = TIMEOUT,
    ) -> client_pb2.StagerListener:
        """Start an HTTP stager listener

        :param host: Host interface to bind the listener to, an empty string will bind to all interfaces
        :type host: str
        :param port: TCP port number to start listener on
        :type port: int
        :param data: Binary data of stage to host on listener
        :type data: bytes
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf StagerListener object
        :rtype: client_pb2.StagerListener
        """
        stage_req = client_pb2.StagerListenerReq(
            Protocol=client_pb2.HTTP, Host=host, Port=port, Data=data
        )
        return await self._stub.StartHTTPStagerListener(stage_req, timeout=timeout)

    async def start_https_stager_listener(
        self,
        host: str,
        port: int,
        data: bytes,
        cert: bytes,
        key: bytes,
        acme: bool,
        timeout: int = TIMEOUT,
    ) -> client_pb2.StagerListener:
        """Start an HTTPS stager listener

        :param host: Host interface to bind the listener to, an empty string will bind to all interfaces
        :type host: str
        :param port: TCP port number to start listener on
        :type port: int
        :param data: Binary data of stage to host on listener
        :type data: bytes
        :param cert: TLS certificate, leave blank to start listener as HTTP
        :type cert: bytes
        :param key: TLS key, leave blank to start listener as HTTP
        :type key: bytes
        :param acme: Automatically provision TLS certificate using ACME (i.e., Let's Encrypt)
        :type acme: bool
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf StagerListener object
        :rtype: client_pb2.StagerListener
        """
        stage_req = client_pb2.StagerListenerReq(
            Protocol=client_pb2.HTTPS,
            Host=host,
            Port=port,
            Data=data,
            Cert=cert,
            Key=key,
            ACME=acme,
        )
        return await self._stub.StartHTTPStagerListener(stage_req, timeout=timeout)

    async def generate_implant(
        self, config: client_pb2.ImplantConfig, timeout: int = 360
    ) -> client_pb2.Generate:
        """Generate a new implant using a given configuration

        :param config: Protobuf ImplantConfig object
        :type config: client_pb2.ImplantConfig
        :param timeout: gRPC timeout, defaults to 360
        :type timeout: int, optional
        :return: Protobuf Generate object containing the generated implant
        :rtype: client_pb2.Generate
        """
        req = client_pb2.GenerateReq(Config=config)
        return await self._stub.Generate(req, timeout=timeout)

    async def regenerate_implant(
        self, implant_name: str, timeout=TIMEOUT
    ) -> client_pb2.Generate:
        """Regenerate an implant binary given the implants "name"

        :param implant_name: The name of the implant to regenerate
        :type implant_name: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf Generate object
        :rtype: client_pb2.Generate
        """
        regenerate = client_pb2.RegenerateReq(ImplantName=implant_name)
        return await self._stub.Regenerate(regenerate, timeout=timeout)

    async def implant_builds(
        self, timeout=TIMEOUT
    ) -> Dict[str, client_pb2.ImplantConfig]:
        """Get information about historical implant builds

        :return: Protobuf Map object, the keys are implant names the values are implant configs
        :rtype: Dict[str, client_pb2.ImplantConfig]
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        """
        builds: client_pb2.ImplantBuilds = await self._stub.ImplantBuilds(
            common_pb2.Empty(), timeout=timeout
        )
        return dict(builds.Configs)

    async def delete_implant_build(self, implant_name: str, timeout=TIMEOUT) -> None:
        """Delete a historical implant build from the server by name

        :param implant_name: The name of the implant build to delete
        :type implant_name: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        """
        delete = client_pb2.DeleteReq(Name=implant_name)
        await self._stub.DeleteImplantBuild(delete, timeout=timeout)

    async def canaries(self, timeout=TIMEOUT) -> List[client_pb2.DNSCanary]:
        """Get a list of canaries that have been generated during implant builds, includes metadata about those canaries

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of Protobuf DNSCanary objects
        :rtype: List[client_pb2.DNSCanary]
        """
        canaries = await self._stub.Canaries(common_pb2.Empty(), timeout=timeout)
        return list(canaries.Canaries)

    async def generate_wg_client_config(
        self, timeout=TIMEOUT
    ) -> client_pb2.WGClientConfig:
        """Generate a new WireGuard client configuration files

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf WGClientConfig object
        :rtype: client_pb2.WGClientConfig
        """
        return await self._stub.GenerateWGClientConfig(
            common_pb2.Empty(), timeout=timeout
        )

    async def generate_wg_ip(self, timeout=TIMEOUT) -> client_pb2.UniqueWGIP:
        """Generate a unique IP address for use with WireGuard

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf UniqueWGIP object
        :rtype: client_pb2.UniqueWGIP
        """
        return await self._stub.GenerateUniqueIP(common_pb2.Empty(), timeout=timeout)

    async def implant_profiles(
        self, timeout=TIMEOUT
    ) -> List[client_pb2.ImplantProfile]:
        """Get a list of all implant configuration profiles on the server

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of Protobuf ImplantProfile objects
        :rtype: List[client_pb2.ImplantProfile]
        """
        profiles = await self._stub.ImplantProfiles(common_pb2.Empty(), timeout=timeout)
        return list(profiles.Profiles)

    async def delete_implant_profile(self, profile_name, timeout=TIMEOUT) -> None:
        """Delete an implant configuration profile by name

        :param profile_name: Name of the profile to delete
        :type profile_name: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        """
        delete = client_pb2.DeleteReq()
        delete.Name = profile_name
        await self._stub.DeleteImplantProfile(delete, timeout=timeout)

    async def save_implant_profile(
        self, profile: client_pb2.ImplantProfile, timeout=TIMEOUT
    ) -> client_pb2.ImplantProfile:
        """Save an implant configuration profile to the server

        :param profile: An implant configuration profile (a Protobuf ImplantProfile object)
        :type profile: client_pb2.ImplantProfile
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf ImplantProfile object
        :rtype: client_pb2.ImplantProfile
        """
        return await self._stub.SaveImplantProfile(profile, timeout=timeout)

    async def generate_msf_stager(
        self,
        arch: str,
        format: str,
        host: str,
        port: int,
        os: str,
        protocol: client_pb2.StageProtocol.ValueType,
        badchars: Optional[Iterable[str]],
        timeout: int = TIMEOUT,
    ) -> client_pb2.MsfStager:
        """Create a Metasploit stager (if available on the server)

        :param arch: CPU architecture
        :type arch: str
        :param format: Binary format (MSF)
        :type format: str
        :param host: LHOST (MSF)
        :type host: str
        :param port: LPORT (MSF)
        :type port: int
        :param os: Operating System (MSF)
        :type os: str
        :param protocol: Stager protocol (Protobuf StageProtocol object)
        :type protocol: client_pb2.StageProtocol
        :param badchars: Bad characters, defaults to []
        :type badchars: list, optional
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf MsfStager object
        :rtype: client_pb2.MsfStager
        """
        stagerReq = client_pb2.MsfStagerReq(
            Arch=arch,
            Format=format,
            Host=host,
            Port=port,
            OS=os,
            Protocol=protocol,
            BadChars=badchars if badchars else [],
        )
        return await self._stub.MsfStage(stagerReq, timeout=timeout)

    async def shellcode(
        self, data: bytes, function_name: str, arguments: str = "", timeout=TIMEOUT
    ) -> client_pb2.ShellcodeRDI:
        """Generate Donut shellcode

        :param data: The DLL file to wrap in a shellcode loader
        :type data: bytes
        :param function_name: Function to call on the DLL
        :type function_name: str
        :param arguments: Arguments to the function called
        :type arguments: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf ShellcodeRDI object
        :rtype: client_pb2.ShellcodeRDI
        """
        shellReq = client_pb2.ShellcodeRDIReq(
            Data=data, FunctionName=function_name, Arguments=arguments
        )
        return await self._stub.ShellcodeRDI(shellReq, timeout=timeout)

    async def websites(self, timeout=TIMEOUT) -> List[client_pb2.Website]:
        """Get a list of websites

        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: List of Protobuf Website objects
        :rtype: List[client_pb2.Website]
        """
        websites = await self._stub.Websites(common_pb2.Empty(), timeout=timeout)
        return list(websites.Websites)

    async def update_website(
        self, website: client_pb2.Website, timeout=TIMEOUT
    ) -> client_pb2.Website:
        """Update an entire website object on the server

        :param website: The updated Protobuf Website object
        :type website: client_pb2.Website
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf Website object
        :rtype: client_pb2.Website
        """
        return await self._stub.Websites(website, timeout=timeout)

    async def remove_website(self, name: str, timeout=TIMEOUT) -> None:
        """Remove an entire website and its content

        :param name: The name of the website to remove
        :type name: str
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        """
        website = client_pb2.Website(Name=name)
        await self._stub.WebsiteRemove(website, timeout=timeout)

    async def add_website_content(
        self,
        name: str,
        web_path: str,
        content_type: str,
        content: bytes,
        timeout: int = TIMEOUT,
    ) -> client_pb2.Website:
        """Add content to a specific website

        :param name: Name of the website to add the content to
        :type name: str
        :param web_path: Bind content to web path
        :type web_path: str
        :param content_type: Specify the Content-type response HTTP header
        :type content_type: str
        :param content: The raw response content
        :type content: bytes
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf Website object
        :rtype: client_pb2.Website
        """
        web = client_pb2.WebContent(
            Path=web_path, ContentType=content_type, Content=content, Size=len(content)
        )

        web_add = client_pb2.WebsiteAddContent(Name=name, Contents={web_path: web})
        return await self._stub.WebsiteAddContent(web_add, timeout=timeout)

    async def update_website_content(
        self,
        name: str,
        web_path: str,
        content_type: str,
        content: bytes,
        timeout: int = TIMEOUT,
    ) -> client_pb2.Website:
        """Update content on a specific website / web path

        :param name: Name of the website to add the content to
        :type name: str
        :param web_path: Bind content to web path
        :type web_path: str
        :param content_type: Specify the Content-type response HTTP header
        :type content_type: str
        :param content: The raw response content
        :type content: bytes
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf Website object
        :rtype: client_pb2.Website
        """
        web = client_pb2.WebContent(
            Path=web_path, ContentType=content_type, Content=content, Size=len(content)
        )

        web_update = client_pb2.WebsiteAddContent(Name=name, Contents={web_path: web})
        return await self._stub.WebsiteUpdateContent(web_update, timeout=timeout)

    async def remove_website_content(
        self, name: str, paths: List[str], timeout=TIMEOUT
    ) -> client_pb2.Website:
        """Remove content from a specific website

        :param name: The name of the website from which to remove the content
        :type name: str
        :param paths: A list of paths to content that should be removed from the website
        :type paths: List[str]
        :param timeout: gRPC timeout, defaults to 60 seconds
        :type timeout: int, optional
        :return: Protobuf Website object
        :rtype: client_pb2.Website
        """
        web = client_pb2.WebsiteRemoveContent(Name=name, Paths=paths)
        return await self._stub.WebsiteRemoveContent(web, timeout=timeout)



"""
    Sliver Implant Framework
    Copyright (C) 2022  Bishop Fox

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
from typing import List, Union

import grpc

from .interactive import BaseInteractiveCommands
from .pb.rpcpb.services_pb2_grpc import SliverRPCStub
from .protobuf import client_pb2, sliver_pb2


class BaseSession:
    """Base class for Session objects.

    :param session: Session protobuf.
    :type session: client_pb2.Session
    :param channel: A gRPC channel.
    :type channel: grpc.Channel
    :param timeout: Timeout in seconds
    :type timeout: int, optional
    """

    def __init__(
        self,
        session: client_pb2.Session,
        channel: grpc.aio.Channel,
        timeout: int = 60,
    ):
        self._channel = channel
        self._session = session
        self._stub = SliverRPCStub(channel)
        self.timeout = timeout

    def _request(self, pb):
        """
        Set request attributes based on current session, I'd prefer to return a generic Request
        object, but protobuf for whatever reason doesn't let you assign this type of field directly.

        `pb` in this case is any protobuf message with a .Request field.

        :param pb: A protobuf request object.
        """
        pb.Request.SessionID = self._session.ID
        pb.Request.Timeout = self.timeout - 1
        return pb

    @property
    def session_id(self) -> str:
        """Session ID"""
        return self._session.ID

    @property
    def name(self) -> str:
        """Session name"""
        return self._session.Name

    @property
    def hostname(self) -> str:
        """Hostname"""
        return self._session.Hostname

    @property
    def uuid(self) -> str:
        """Session UUID"""
        return self._session.UUID

    @property
    def username(self) -> str:
        """Username"""
        return self._session.Username

    @property
    def uid(self) -> str:
        """User ID"""
        return self._session.UID

    @property
    def gid(self) -> str:
        """Group ID"""
        return self._session.GID

    @property
    def os(self) -> str:
        """Operating system"""
        return self._session.OS

    @property
    def arch(self) -> str:
        """Architecture"""
        return self._session.Arch

    @property
    def transport(self) -> str:
        """Transport Method"""
        return self._session.Transport

    @property
    def remote_address(self) -> str:
        """Remote address"""
        return self._session.RemoteAddress

    @property
    def pid(self) -> int:
        """Process ID"""
        return self._session.PID

    @property
    def filename(self) -> str:
        """Implant filename"""
        return self._session.Filename

    @property
    def last_checkin(self) -> int:
        """Last check in"""
        return self._session.LastCheckin

    @property
    def active_c2(self) -> str:
        """Active C2"""
        return self._session.ActiveC2

    @property
    def version(self) -> str:
        """Version"""
        return self._session.Version

    @property
    def is_dead(self) -> bool:
        """Is dead"""
        return self._session.IsDead

    @property
    def reconnect_interval(self) -> int:
        """Reconnect interval"""
        return self._session.ReconnectInterval

    @property
    def proxy_url(self) -> str:
        """Proxy URL"""

        return self._session.ProxyURL


class InteractiveSession(BaseSession, BaseInteractiveCommands):
    """Session only commands"""

    async def pivot_listeners(self) -> List[sliver_pb2.PivotListener]:
        """List C2 pivots

        :return: Protobuf PivotListener list
        :rtype: List[sliver_pb2.PivotListener]
        """
        pivots = await self._stub.PivotSessionListeners(
            self._request(sliver_pb2.PivotListenersReq()), timeout=self.timeout
        )
        return list(pivots.Listeners)

    async def start_service(
        self, name: str, description: str, exe: str, hostname: str, arguments: str
    ) -> sliver_pb2.ServiceInfo:
        """Create and start a Windows service (Windows only)

        :param name: Name of the service
        :type name: str
        :param description: Service description
        :type description: str
        :param exe: Path to the service .exe file
        :type exe: str
        :param hostname: Hostname
        :type hostname: str
        :param arguments: Arguments to start the service with
        :type arguments: str
        :return: Protobuf ServiceInfo object
        :rtype: sliver_pb2.ServiceInfo
        """
        svc = sliver_pb2.StartServiceReq()
        svc.ServiceName = name
        svc.ServiceDescription = description
        svc.BinPath = exe
        svc.Hostname = hostname
        svc.Arguments = arguments
        return await self._stub.StartService(self._request(svc), timeout=self.timeout)

    async def stop_service(self, name: str, hostname: str) -> sliver_pb2.ServiceInfo:
        """Stop a Windows service (Windows only)

        :param name: Name of the servie
        :type name: str
        :param hostname: Hostname
        :type hostname: str
        :return: Protobuf ServiceInfo object
        :rtype: sliver_pb2.ServiceInfo
        """
        svc = sliver_pb2.StopServiceReq()
        svc.ServiceInfo.ServiceName = name
        svc.ServiceInfo.Hostname = hostname
        return await self._stub.StopService(self._request(svc), timeout=self.timeout)

    async def remove_service(self, name: str, hostname: str) -> sliver_pb2.ServiceInfo:
        """Remove a Windows service (Windows only)

        :param name: Name of the service
        :type name: str
        :param hostname: Hostname
        :type hostname: str
        :return: Protobuf ServiceInfo object
        :rtype: sliver_pb2.ServiceInfo
        """
        svc = sliver_pb2.StopServiceReq()
        svc.ServiceInfo.ServiceName = name
        svc.ServiceInfo.Hostname = hostname
        return await self._stub.RemoveService(self._request(svc), timeout=self.timeout)

    async def backdoor(
        self, remote_path: str, profile_name: str
    ) -> sliver_pb2.Backdoor:
        """Backdoor a remote binary by injecting a Sliver payload into the executable using a code cave

        :param remote_path: Remote path to an executable to backdoor
        :type remote_path: str
        :param profile_name: Implant profile name to inject into the binary
        :type profile_name: str
        :return: Protobuf Backdoor object
        :rtype: sliver_pb2.Backdoor
        """
        backdoor = sliver_pb2.BackdoorReq()
        backdoor.FilePath = remote_path
        backdoor.ProfileName = profile_name
        return await self._stub.Backdoor(self._request(backdoor), timeout=self.timeout)



import functools
import inspect
import logging
import os


def inspect_debug(obj):
    # If we are in dev environment and the result is a class, inspect the class
    if os.getenv("HATCH_ENV_ACTIVE"):
        from rich import inspect as rinspect

        rinspect(obj)


def log_return(level: str):
    """Logs return value of function or coroutine at level specified"""

    def is_async(func):
        """Determine if function or generator is async"""
        return (
            inspect.iscoroutinefunction(func)
            or inspect.isasyncgenfunction(func)
            or inspect.isasyncgen(func)
            or inspect.iscoroutine(func)
        )

    def decorator(func):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            result = None
            if is_async(func):
                result = await func(*args, **kwargs)
            else:
                result = func(*args, **kwargs)
            logging.log(
                logging.getLevelName(level.upper()),
                f"{func.__qualname__} returned:\n{result}",
            )

            # If we are in dev environment and the result is a class, inspect the class
            inspect_debug(result)

            return result

        return wrapper

    return decorator



"""
    Sliver Implant Framework
    Copyright (C) 2022  Bishop Fox

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from typing import List, Optional

from sliver.pb.commonpb import common_pb2

from ._protocols import InteractiveObject
from .protobuf import client_pb2, sliver_pb2


class BaseInteractiveCommands:
    async def ping(self: InteractiveObject) -> sliver_pb2.Ping:
        """Send a round trip message to the implant (does NOT use ICMP)

        :return: Protobuf ping object
        :rtype: sliver_pb2.Ping
        """
        return await self._stub.Ping(
            self._request(sliver_pb2.Ping()), timeout=self.timeout
        )

    async def ps(self: InteractiveObject) -> List[common_pb2.Process]:
        """List the processes of the remote system

        :return: Ps protobuf object
        :rtype: List[common_pb2.Process]
        """
        ps = sliver_pb2.PsReq()
        processes = await self._stub.Ps(self._request(ps), timeout=self.timeout)
        return list(processes.Processes)

    async def terminate(
        self: InteractiveObject, pid: int, force=False
    ) -> sliver_pb2.Terminate:
        """Terminate a remote process

        :param pid: The process ID to terminate.
        :type pid: int
        :param force: Force termination of the process, defaults to False
        :type force: bool, optional
        :return: Protobuf terminate object
        :rtype: sliver_pb2.Terminate
        """
        terminator = sliver_pb2.TerminateReq(Pid=pid, Force=force)
        return await self._stub.Terminate(
            self._request(terminator), timeout=self.timeout
        )

    async def ifconfig(self: InteractiveObject) -> sliver_pb2.Ifconfig:
        """Get network interface configuration information about the remote system

        :return: Protobuf ifconfig object
        :rtype: sliver_pb2.Ifconfig
        """
        return await self._stub.Ifconfig(
            self._request(sliver_pb2.IfconfigReq()), timeout=self.timeout
        )

    async def netstat(
        self: InteractiveObject,
        tcp: bool,
        udp: bool,
        ipv4: bool,
        ipv6: bool,
        listening=True,
    ) -> sliver_pb2.Netstat:
        """Get information about network connections on the remote system.

        :param tcp: Get TCP information
        :type tcp: bool
        :param udp: Get UDP information
        :type udp: bool
        :param ipv4: Get IPv4 connection information
        :type ipv4: bool
        :param ipv6: Get IPv6 connection information
        :type ipv6: bool
        :param listening: Get listening connection information, defaults to True
        :type listening: bool, optional
        :return: Protobuf netstat object
        :rtype: List[sliver_pb2.SockTabEntry]
        """
        net = sliver_pb2.NetstatReq(
            TCP=tcp, UDP=udp, IP4=ipv4, IP6=ipv6, Listening=listening
        )
        return await self._stub.Netstat(self._request(net), timeout=self.timeout)

    async def ls(self: InteractiveObject, remote_path: str = ".") -> sliver_pb2.Ls:
        """Get a directory listing from the remote system

        :param remote_path: Remote path
        :type remote_path: str
        :return: Protobuf ls object
        :rtype: sliver_pb2.Ls
        """
        ls = sliver_pb2.LsReq(Path=remote_path)
        return await self._stub.Ls(self._request(ls), timeout=self.timeout)

    async def cd(self: InteractiveObject, remote_path: str) -> sliver_pb2.Pwd:
        """Change the current working directory of the implant

        :param remote_path: Remote path
        :type remote_path: str
        :return: Protobuf pwd object
        :rtype: sliver_pb2.Pwd
        """
        cd = sliver_pb2.CdReq(Path=remote_path)
        return await self._stub.Cd(self._request(cd), timeout=self.timeout)

    async def pwd(self: InteractiveObject) -> sliver_pb2.Pwd:
        """Get the implant's current working directory

        :return: Protobuf pwd object
        :rtype: sliver_pb2.Pwd
        """
        pwd = sliver_pb2.PwdReq()
        return await self._stub.Pwd(self._request(pwd), timeout=self.timeout)

    async def rm(
        self: InteractiveObject, remote_path: str, recursive=False, force=False
    ) -> sliver_pb2.Rm:
        """Remove a directory or file(s)

        :param remote_path: Remote path
        :type remote_path: str
        :param recursive: Recursively remove file(s), defaults to False
        :type recursive: bool, optional
        :param force: Forcefully remove the file(s), defaults to False
        :type force: bool, optional
        :return: Protobuf rm object
        :rtype: sliver_pb2.Rm
        """
        rm = sliver_pb2.RmReq(Path=remote_path, Recursive=recursive, Force=force)
        return await self._stub.Rm(self._request(rm), timeout=self.timeout)

    async def mkdir(self: InteractiveObject, remote_path: str) -> sliver_pb2.Mkdir:
        """Make a directory on the remote file system

        :param remote_path: Directory to create
        :type remote_path: str
        :return: Protobuf Mkdir object
        :rtype: sliver_pb2.Mkdir
        """
        make = sliver_pb2.MkdirReq(Path=remote_path)
        return await self._stub.Mkdir(self._request(make), timeout=self.timeout)

    async def download(
        self: InteractiveObject, remote_path: str, recurse: bool = False
    ) -> sliver_pb2.Download:
        """Download a file or directory from the remote file system

        :param remote_path: File to download
        :type remote_path: str
        :param recurse: Download all files in a directory
        :type recurse: bool
        :return: Protobuf Download object
        :rtype: sliver_pb2.Download
        """
        download = sliver_pb2.DownloadReq(Path=remote_path, Recurse=recurse)
        return await self._stub.Download(self._request(download), timeout=self.timeout)

    async def upload(
        self: InteractiveObject,
        remote_path: str,
        data: bytes,
        is_ioc: bool = False,
    ) -> sliver_pb2.Upload:
        """Write data to specified path on remote file system

        :param remote_path: Remote path
        :type remote_path: str
        :param data: Data to write
        :type data: bytes
        :param is_ioc: Data is an indicator of compromise, defaults to False
        :type is_ioc: bool, optional
        :return: Protobuf Upload object
        :rtype: sliver_pb2.Upload
        """
        upload = sliver_pb2.UploadReq(Path=remote_path, Data=data, IsIOC=is_ioc)
        return await self._stub.Upload(self._request(upload), timeout=self.timeout)

    async def process_dump(self: InteractiveObject, pid: int) -> sliver_pb2.ProcessDump:
        """Dump a remote process' memory

        :param pid: PID of the process to dump
        :type pid: int
        :return: Protobuf ProcessDump object
        :rtype: sliver_pb2.ProcessDump
        """
        procdump = sliver_pb2.ProcessDumpReq(Pid=pid)
        return await self._stub.ProcessDump(
            self._request(procdump), timeout=self.timeout
        )

    async def run_as(
        self: InteractiveObject, username: str, process_name: str, args: str
    ) -> sliver_pb2.RunAs:
        """Run a command as another user on the remote system

        :param username: User to run process as
        :type username: str
        :param process_name: Process to execute
        :type process_name: str
        :param args: Arguments to process
        :type args: str
        :return: Protobuf RunAs object
        :rtype: sliver_pb2.RunAs
        """
        run_as = sliver_pb2.RunAsReq(
            Username=username, ProcessName=process_name, Args=args
        )
        return await self._stub.RunAs(self._request(run_as), timeout=self.timeout)

    async def impersonate(
        self: InteractiveObject, username: str
    ) -> sliver_pb2.Impersonate:
        """Impersonate a user using tokens (Windows only)

        :param username: User to impersonate
        :type username: str
        :return: Protobuf Impersonate object
        :rtype: sliver_pb2.Impersonate
        """
        impersonate = sliver_pb2.ImpersonateReq(Username=username)
        return await self._stub.Impersonate(
            self._request(impersonate), timeout=self.timeout
        )

    async def revert_to_self(self: InteractiveObject) -> sliver_pb2.RevToSelf:
        """Revert to self from impersonation context

        :return: Protobuf RevToSelf object
        :rtype: sliver_pb2.RevToSelf
        """
        return await self._stub.RevToSelf(
            self._request(sliver_pb2.RevToSelfReq()), timeout=self.timeout
        )

    async def get_system(
        self: InteractiveObject, hosting_process: str, config: client_pb2.ImplantConfig
    ) -> sliver_pb2.GetSystem:
        """Attempt to get SYSTEM (Windows only)

        :param hosting_process: Hosting process to attempt gaining privileges
        :type hosting_process: str
        :param config: Implant configuration to be injected into the hosting process
        :type config: client_pb2.ImplantConfig
        :return: Protobuf GetSystem object
        :rtype: sliver_pb2.GetSystem
        """
        system = client_pb2.GetSystemReq(HostingProcess=hosting_process, Config=config)
        return await self._stub.GetSystem(self._request(system), timeout=self.timeout)

    async def execute_shellcode(
        self: InteractiveObject, data: bytes, rwx: bool, pid: int, encoder=""
    ) -> sliver_pb2.Task:
        """Execute shellcode in-memory

        :param data: Shellcode buffer
        :type data: bytes
        :param rwx: Enable/disable RWX pages
        :type rwx: bool
        :param pid: Process ID to inject shellcode into
        :type pid: int
        :param encoder: Encoder ('', 'gzip'), defaults to ''
        :type encoder: str, optional
        :return: Protobuf Task object
        :rtype: sliver_pb2.Task
        """
        task = sliver_pb2.TaskReq(Encoder=encoder, Data=data, RWXPages=rwx, Pid=pid)
        return await self._stub.Task(self._request(task), timeout=self.timeout)

    async def msf(
        self: InteractiveObject,
        payload: str,
        lhost: str,
        lport: int,
        encoder: str,
        iterations: int,
    ) -> None:
        """Execute Metasploit payload on remote system, the payload will be generated by the server
        based on the parameters to this function. The server must be configured with Metasploit.

        :param payload: Payload to generate
        :type payload: str
        :param lhost: Metasploit LHOST parameter
        :type lhost: str
        :param lport: Metasploit LPORT parameter
        :type lport: int
        :param encoder: Metasploit encoder
        :type encoder: str
        :param iterations: Iterations for Metasploit encoder
        :type iterations: int
        """
        msf = client_pb2.MSFReq()
        msf.Payload = payload
        msf.LHost = lhost
        msf.LPort = lport
        msf.Encoder = encoder
        msf.Iterations = iterations
        return await self._stub.Msf(self._request(msf), timeout=self.timeout)

    async def msf_remote(
        self: InteractiveObject,
        payload: str,
        lhost: str,
        lport: int,
        encoder: str,
        iterations: int,
        pid: int,
    ) -> None:
        """Execute Metasploit payload in a remote process, the payload will be generated by the server
        based on the parameters to this function. The server must be configured with Metasploit.

        :param payload: Payload to generate
        :type payload: str
        :param lhost: Metasploit LHOST parameter
        :type lhost: str
        :param lport: Metasploit LPORT parameter
        :type lport: int
        :param encoder: Metasploit encoder
        :type encoder: str
        :param iterations: Iterations for Metasploit encoder
        :type iterations: int
        :param pid: Process ID to inject the payload into
        :type pid: int
        """
        msf = client_pb2.MSFRemoteReq()
        msf.Payload = payload
        msf.LHost = lhost
        msf.LPort = lport
        msf.Encoder = encoder
        msf.Iterations = iterations
        msf.PID = pid
        return await self._stub.Msf(self._request(msf), timeout=self.timeout)

    async def execute_assembly(
        self: InteractiveObject,
        assembly: bytes,
        arguments: str,
        process: str,
        is_dll: bool,
        arch: str,
        class_name: str,
        method: str,
        app_domain: str,
    ) -> sliver_pb2.ExecuteAssembly:
        """Execute a .NET assembly in-memory on the remote system

        :param assembly: A buffer of the .NET assembly to execute
        :type assembly: bytes
        :param arguments: Arguments to the .NET assembly
        :type arguments: str
        :param process: Process to execute assembly
        :type process: str
        :param is_dll: Is assembly a DLL
        :type is_dll: bool
        :param arch: Assembly architecture
        :type arch: str
        :param class_name: Class name of the assembly
        :type class_name: str
        :param method: Method to execute
        :type method: str
        :param app_domain: AppDomain
        :type app_domain: str
        :return: Protobuf ExecuteAssembly object
        :rtype: sliver_pb2.ExecuteAssembly
        """
        asm = sliver_pb2.ExecuteAssemblyReq()
        asm.Assembly = assembly
        asm.Arguments = arguments
        asm.Process = process
        asm.IsDLL = is_dll
        asm.Arch = arch
        asm.ClassName = class_name
        asm.AppDomain = app_domain
        return await self._stub.ExecuteAssembly(
            self._request(asm), timeout=self.timeout
        )

    async def migrate(
        self: InteractiveObject, pid: int, config: client_pb2.ImplantConfig
    ) -> sliver_pb2.Migrate:
        """Migrate implant to another process

        :param pid: Process ID to inject implant into
        :type pid: int
        :param config: Implant configuration to inject into the remote process
        :type config: client_pb2.ImplantConfig
        :return: Protobuf Migrate object
        :rtype: sliver_pb2.Migrate
        """
        migrate = client_pb2.MigrateReq()
        migrate.Pid = pid
        migrate.Config.CopyFrom(config)
        return await self._stub.Migrate(self._request(migrate), timeout=self.timeout)

    async def execute(
        self: InteractiveObject,
        exe: str,
        args: Optional[List[str]],
        output: bool = True,
    ) -> sliver_pb2.Execute:
        """Execute a command/subprocess on the remote system

        :param exe: Command/subprocess to execute
        :type exe: str
        :param args: Arguments to the command/subprocess
        :type args: List[str]
        :param output: Enable capturing command/subprocess stdout
        :type output: bool
        :return: Protobuf Execute object
        :rtype: sliver_pb2.Execute
        """
        if not args:
            args = []
        execute_req = sliver_pb2.ExecuteReq(Path=exe, Args=args, Output=output)
        return await self._stub.Execute(
            self._request(execute_req), timeout=self.timeout
        )

    async def sideload(
        self: InteractiveObject,
        data: bytes,
        process_name: str,
        arguments: str,
        entry_point: str,
        kill: bool,
    ) -> sliver_pb2.Sideload:
        """Sideload a shared library into a remote process using a platform specific in-memory loader (Windows, MacOS, Linux only)

        :param data: Shared library raw bytes
        :type data: bytes
        :param process_name: Process name to sideload library into
        :type process_name: str
        :param arguments: Arguments to the shared library
        :type arguments: str
        :param entry_point: Entrypoint of the shared library
        :type entry_point: str
        :param kill: Kill normal execution of the process when side loading the shared library
        :type kill: bool
        :return: Protobuf Sideload object
        :rtype: sliver_pb2.Sideload
        """
        side = sliver_pb2.SideloadReq(
            Data=data,
            ProcessName=process_name,
            Args=arguments,
            EntryPoint=entry_point,
            Kill=kill,
        )
        return await self._stub.Sideload(self._request(side), timeout=self.timeout)

    async def spawn_dll(
        self: InteractiveObject,
        data: bytes,
        process_name: str,
        arguments: str,
        entry_point: str,
        kill: bool,
    ) -> sliver_pb2.SpawnDll:
        """Spawn a DLL on the remote system from memory (Windows only)

        :param data: DLL raw bytes
        :type data: bytes
        :param process_name: Process name to spawn DLL into
        :type process_name: str
        :param arguments: Arguments to the DLL
        :type arguments: str
        :param entry_point: Entrypoint of the DLL
        :type entry_point: str
        :param kill: Kill normal execution of the remote process when spawing the DLL
        :type kill: bool
        :return: Protobuf SpawnDll object
        :rtype: sliver_pb2.SpawnDll
        """
        spawn = sliver_pb2.InvokeSpawnDllReq(
            Data=data,
            ProcessName=process_name,
            Args=arguments,
            EntryPoint=entry_point,
            Kill=kill,
        )
        return await self._stub.SpawnDll(self._request(spawn), timeout=self.timeout)

    async def screenshot(self: InteractiveObject) -> sliver_pb2.Screenshot:
        """Take a screenshot of the remote system, screenshot data is PNG formatted

        :return: Protobuf Screenshot object
        :rtype: sliver_pb2.Screenshot
        """
        return await self._stub.Screenshot(
            self._request(sliver_pb2.ScreenshotReq()), timeout=self.timeout
        )

    async def make_token(
        self: InteractiveObject, username: str, password: str, domain: str
    ) -> sliver_pb2.MakeToken:
        """Make a Windows user token from a valid login (Windows only)

        :param username: Username
        :type username: str
        :param password: Password
        :type password: str
        :param domain: Domain
        :type domain: str
        :return: Protobuf MakeToken object
        :rtype: sliver_pb2.MakeToken
        """
        make = sliver_pb2.MakeTokenReq(
            Username=username, Password=password, Domain=domain
        )
        return await self._stub.MakeToken(self._request(make), timeout=self.timeout)

    async def get_env(self: InteractiveObject, name: str) -> sliver_pb2.EnvInfo:
        """Get an environment variable

        :param name: Name of the variable
        :type name: str
        :return: Protobuf EnvInfo object
        :rtype: sliver_pb2.EnvInfo
        """
        env = sliver_pb2.EnvReq(Name=name)
        return await self._stub.GetEnv(self._request(env), timeout=self.timeout)

    async def set_env(
        self: InteractiveObject, key: str, value: str
    ) -> sliver_pb2.SetEnv:
        """Set an environment variable

        :param name: Name of the environment variable
        :type name: str
        :param value: Value of the environment variable
        :type value: str
        :return: Protobuf SetEnv object
        :rtype: sliver_pb2.SetEnv
        """
        env_var = common_pb2.EnvVar(Key=key, Value=value)
        env_req = sliver_pb2.SetEnvReq(Variable=env_var)
        return await self._stub.SetEnv(self._request(env_req), timeout=self.timeout)

    async def unset_env(self: InteractiveObject, key: str) -> sliver_pb2.UnsetEnv:
        """Unset an environment variable

        :param value: Value of the environment variable
        :type value: str
        :return: Protobuf SetEnv object
        :rtype: sliver_pb2.SetEnv
        """
        env = sliver_pb2.UnsetEnvReq(Name=key)
        return await self._stub.UnsetEnv(self._request(env), timeout=self.timeout)

    async def registry_read(
        self: InteractiveObject, hive: str, reg_path: str, key: str, hostname: str
    ) -> sliver_pb2.RegistryRead:
        """Read a value from the remote system's registry (Windows only)

        :param hive: Registry hive to read value from
        :type hive: str
        :param reg_path: Path to registry key to read
        :type reg_path: str
        :param key: Key name to read
        :type key: str
        :param hostname: Hostname
        :type hostname: str
        :return: Protobuf RegistryRead object
        :rtype: sliver_pb2.RegistryRead
        """
        reg = sliver_pb2.RegistryReadReq()
        reg.Hive = hive
        reg.Path = reg_path
        reg.Key = key
        reg.Hostname = hostname
        return await self._stub.RegistryRead(self._request(reg), timeout=self.timeout)

    async def registry_write(
        self: InteractiveObject,
        hive: str,
        reg_path: str,
        key: str,
        hostname: str,
        string_value: str,
        byte_value: bytes,
        dword_value: int,
        qword_value: int,
        reg_type: sliver_pb2.RegistryType.ValueType,
    ) -> sliver_pb2.RegistryWrite:
        """Write a value to the remote system's registry (Windows only)

        :param hive: Registry hive to write the key/value to
        :type hive: str
        :param reg_path: Registry path to write to
        :type reg_path: str
        :param key: Registry key to write to
        :type key: str
        :param hostname: Hostname
        :type hostname: str
        :param string_value: String value to write (ignored for non-string key)
        :type string_value: str
        :param byte_value: Byte value to write (ignored for non-byte key)
        :type byte_value: bytes
        :param dword_value: DWORD value to write (ignored for non-DWORD key)
        :type dword_value: int
        :param qword_value: QWORD value to write (ignored for non-QWORD key)
        :type qword_value: int
        :param reg_type: Type of registry key to write
        :type reg_type: sliver_pb2.RegistryType
        :return: Protobuf RegistryWrite object
        :rtype: sliver_pb2.RegistryWrite
        """
        reg = sliver_pb2.RegistryWriteReq(
            Hive=hive,
            Path=reg_path,
            Key=key,
            Hostname=hostname,
            StringValue=string_value,
            ByteValue=byte_value,
            DWordValue=dword_value,
            QWordValue=qword_value,
            Type=int(reg_type),
        )
        reg.Hive = hive
        reg.Path = reg_path
        reg.Key = key
        reg.Hostname = hostname
        reg.StringValue = string_value
        reg.ByteValue = byte_value
        reg.DWordValue = dword_value
        reg.QWordValue = qword_value
        reg.Type = reg_type

        return await self._stub.RegistryWrite(self._request(reg), timeout=self.timeout)

    async def registry_create_key(
        self: InteractiveObject, hive: str, reg_path: str, key: str, hostname: str
    ) -> sliver_pb2.RegistryCreateKey:
        """Create a registry key on the remote system (Windows only)

        :param hive: Registry hive to create key in
        :type hive: str
        :param reg_path: Registry path to create key in
        :type reg_path: str
        :param key: Key name
        :type key: str
        :param hostname: Hostname
        :type hostname: str
        :return: Protobuf RegistryCreateKey object
        :rtype: sliver_pb2.RegistryCreateKey
        """
        reg = sliver_pb2.RegistryCreateKeyReq(
            Hive=hive, Path=reg_path, Key=key, Hostname=hostname
        )
        return await self._stub.RegistryCreateKey(
            self._request(reg), timeout=self.timeout
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rpcpb/services.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..clientpb import client_pb2 as clientpb_dot_client__pb2
from ..commonpb import common_pb2 as commonpb_dot_common__pb2
from ..sliverpb import sliver_pb2 as sliverpb_dot_sliver__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b"\n\x14rpcpb/services.proto\x12\x05rpcpb\x1a\x15\x63ommonpb/common.proto\x1a\x15sliverpb/sliver.proto\x1a\x15\x63lientpb/client.proto2\xe4<\n\tSliverRPC\x12\x30\n\nGetVersion\x12\x0f.commonpb.Empty\x1a\x11.clientpb.Version\x12\x34\n\x0cGetOperators\x12\x0f.commonpb.Empty\x1a\x13.clientpb.Operators\x12*\n\x04Kill\x12\x11.sliverpb.KillReq\x1a\x0f.commonpb.Empty\x12>\n\x0bReconfigure\x12\x18.sliverpb.ReconfigureReq\x1a\x15.sliverpb.Reconfigure\x12.\n\x06Rename\x12\x13.clientpb.RenameReq\x1a\x0f.commonpb.Empty\x12\x32\n\x0bGetSessions\x12\x0f.commonpb.Empty\x1a\x12.clientpb.Sessions\x12\x30\n\nGetBeacons\x12\x0f.commonpb.Empty\x1a\x11.clientpb.Beacons\x12/\n\tGetBeacon\x12\x10.clientpb.Beacon\x1a\x10.clientpb.Beacon\x12-\n\x08RmBeacon\x12\x10.clientpb.Beacon\x1a\x0f.commonpb.Empty\x12\x39\n\x0eGetBeaconTasks\x12\x10.clientpb.Beacon\x1a\x15.clientpb.BeaconTasks\x12\x42\n\x14GetBeaconTaskContent\x12\x14.clientpb.BeaconTask\x1a\x14.clientpb.BeaconTask\x12>\n\x10\x43\x61ncelBeaconTask\x12\x14.clientpb.BeaconTask\x1a\x14.clientpb.BeaconTask\x12\x33\n\x0cMonitorStart\x12\x0f.commonpb.Empty\x1a\x12.commonpb.Response\x12/\n\x0bMonitorStop\x12\x0f.commonpb.Empty\x1a\x0f.commonpb.Empty\x12*\n\x07GetJobs\x12\x0f.commonpb.Empty\x1a\x0e.clientpb.Jobs\x12\x32\n\x07KillJob\x12\x14.clientpb.KillJobReq\x1a\x11.clientpb.KillJob\x12\x46\n\x11StartMTLSListener\x12\x19.clientpb.MTLSListenerReq\x1a\x16.clientpb.MTLSListener\x12@\n\x0fStartWGListener\x12\x17.clientpb.WGListenerReq\x1a\x14.clientpb.WGListener\x12\x43\n\x10StartDNSListener\x12\x18.clientpb.DNSListenerReq\x1a\x15.clientpb.DNSListener\x12G\n\x12StartHTTPSListener\x12\x19.clientpb.HTTPListenerReq\x1a\x16.clientpb.HTTPListener\x12\x46\n\x11StartHTTPListener\x12\x19.clientpb.HTTPListenerReq\x1a\x16.clientpb.HTTPListener\x12O\n\x16StartTCPStagerListener\x12\x1b.clientpb.StagerListenerReq\x1a\x18.clientpb.StagerListener\x12P\n\x17StartHTTPStagerListener\x12\x1b.clientpb.StagerListenerReq\x1a\x18.clientpb.StagerListener\x12)\n\x07LootAdd\x12\x0e.clientpb.Loot\x1a\x0e.clientpb.Loot\x12)\n\x06LootRm\x12\x0e.clientpb.Loot\x1a\x0f.commonpb.Empty\x12,\n\nLootUpdate\x12\x0e.clientpb.Loot\x1a\x0e.clientpb.Loot\x12-\n\x0bLootContent\x12\x0e.clientpb.Loot\x1a\x0e.clientpb.Loot\x12-\n\x07LootAll\x12\x0f.commonpb.Empty\x1a\x11.clientpb.AllLoot\x12.\n\tLootAllOf\x12\x0e.clientpb.Loot\x1a\x11.clientpb.AllLoot\x12,\n\x05Hosts\x12\x0f.commonpb.Empty\x1a\x12.clientpb.AllHosts\x12&\n\x04Host\x12\x0e.clientpb.Host\x1a\x0e.clientpb.Host\x12)\n\x06HostRm\x12\x0e.clientpb.Host\x1a\x0f.commonpb.Empty\x12+\n\tHostIOCRm\x12\r.clientpb.IOC\x1a\x0f.commonpb.Empty\x12\x35\n\x08Generate\x12\x15.clientpb.GenerateReq\x1a\x12.clientpb.Generate\x12J\n\x10GenerateExternal\x12\x15.clientpb.GenerateReq\x1a\x1f.clientpb.ExternalImplantConfig\x12M\n\x19GenerateExternalSaveBuild\x12\x1f.clientpb.ExternalImplantBinary\x1a\x0f.commonpb.Empty\x12\\\n GenerateExternalGetImplantConfig\x12\x17.clientpb.ImplantConfig\x1a\x1f.clientpb.ExternalImplantConfig\x12\x39\n\nRegenerate\x12\x17.clientpb.RegenerateReq\x1a\x12.clientpb.Generate\x12\x39\n\rImplantBuilds\x12\x0f.commonpb.Empty\x1a\x17.clientpb.ImplantBuilds\x12:\n\x12\x44\x65leteImplantBuild\x12\x13.clientpb.DeleteReq\x1a\x0f.commonpb.Empty\x12/\n\x08\x43\x61naries\x12\x0f.commonpb.Empty\x1a\x12.clientpb.Canaries\x12\x43\n\x16GenerateWGClientConfig\x12\x0f.commonpb.Empty\x1a\x18.clientpb.WGClientConfig\x12\x39\n\x10GenerateUniqueIP\x12\x0f.commonpb.Empty\x1a\x14.clientpb.UniqueWGIP\x12=\n\x0fImplantProfiles\x12\x0f.commonpb.Empty\x1a\x19.clientpb.ImplantProfiles\x12<\n\x14\x44\x65leteImplantProfile\x12\x13.clientpb.DeleteReq\x1a\x0f.commonpb.Empty\x12H\n\x12SaveImplantProfile\x12\x18.clientpb.ImplantProfile\x1a\x18.clientpb.ImplantProfile\x12\x37\n\x08MsfStage\x12\x16.clientpb.MsfStagerReq\x1a\x13.clientpb.MsfStager\x12\x41\n\x0cShellcodeRDI\x12\x19.clientpb.ShellcodeRDIReq\x1a\x16.clientpb.ShellcodeRDI\x12\x32\n\x0bGetCompiler\x12\x0f.commonpb.Empty\x1a\x12.clientpb.Compiler\x12K\n\x10ShellcodeEncoder\x12\x1c.clientpb.ShellcodeEncodeReq\x1a\x19.clientpb.ShellcodeEncode\x12\x45\n\x13ShellcodeEncoderMap\x12\x0f.commonpb.Empty\x1a\x1d.clientpb.ShellcodeEncoderMap\x12/\n\x08Websites\x12\x0f.commonpb.Empty\x1a\x12.clientpb.Websites\x12/\n\x07Website\x12\x11.clientpb.Website\x1a\x11.clientpb.Website\x12\x33\n\rWebsiteRemove\x12\x11.clientpb.Website\x1a\x0f.commonpb.Empty\x12\x43\n\x11WebsiteAddContent\x12\x1b.clientpb.WebsiteAddContent\x1a\x11.clientpb.Website\x12\x46\n\x14WebsiteUpdateContent\x12\x1b.clientpb.WebsiteAddContent\x1a\x11.clientpb.Website\x12I\n\x14WebsiteRemoveContent\x12\x1e.clientpb.WebsiteRemoveContent\x1a\x11.clientpb.Website\x12&\n\x04Ping\x12\x0e.sliverpb.Ping\x1a\x0e.sliverpb.Ping\x12#\n\x02Ps\x12\x0f.sliverpb.PsReq\x1a\x0c.sliverpb.Ps\x12\x38\n\tTerminate\x12\x16.sliverpb.TerminateReq\x1a\x13.sliverpb.Terminate\x12\x35\n\x08Ifconfig\x12\x15.sliverpb.IfconfigReq\x1a\x12.sliverpb.Ifconfig\x12\x32\n\x07Netstat\x12\x14.sliverpb.NetstatReq\x1a\x11.sliverpb.Netstat\x12#\n\x02Ls\x12\x0f.sliverpb.LsReq\x1a\x0c.sliverpb.Ls\x12$\n\x02\x43\x64\x12\x0f.sliverpb.CdReq\x1a\r.sliverpb.Pwd\x12&\n\x03Pwd\x12\x10.sliverpb.PwdReq\x1a\r.sliverpb.Pwd\x12#\n\x02Mv\x12\x0f.sliverpb.MvReq\x1a\x0c.sliverpb.Mv\x12#\n\x02Rm\x12\x0f.sliverpb.RmReq\x1a\x0c.sliverpb.Rm\x12,\n\x05Mkdir\x12\x12.sliverpb.MkdirReq\x1a\x0f.sliverpb.Mkdir\x12\x35\n\x08\x44ownload\x12\x15.sliverpb.DownloadReq\x1a\x12.sliverpb.Download\x12/\n\x06Upload\x12\x13.sliverpb.UploadReq\x1a\x10.sliverpb.Upload\x12>\n\x0bProcessDump\x12\x18.sliverpb.ProcessDumpReq\x1a\x15.sliverpb.ProcessDump\x12,\n\x05RunAs\x12\x12.sliverpb.RunAsReq\x1a\x0f.sliverpb.RunAs\x12>\n\x0bImpersonate\x12\x18.sliverpb.ImpersonateReq\x1a\x15.sliverpb.Impersonate\x12\x38\n\tRevToSelf\x12\x16.sliverpb.RevToSelfReq\x1a\x13.sliverpb.RevToSelf\x12\x38\n\tGetSystem\x12\x16.clientpb.GetSystemReq\x1a\x13.sliverpb.GetSystem\x12)\n\x04Task\x12\x11.sliverpb.TaskReq\x1a\x0e.sliverpb.Task\x12'\n\x03Msf\x12\x10.clientpb.MSFReq\x1a\x0e.sliverpb.Task\x12\x33\n\tMsfRemote\x12\x16.clientpb.MSFRemoteReq\x1a\x0e.sliverpb.Task\x12J\n\x0f\x45xecuteAssembly\x12\x1c.sliverpb.ExecuteAssemblyReq\x1a\x19.sliverpb.ExecuteAssembly\x12\x32\n\x07Migrate\x12\x14.clientpb.MigrateReq\x1a\x11.sliverpb.Migrate\x12\x32\n\x07\x45xecute\x12\x14.sliverpb.ExecuteReq\x1a\x11.sliverpb.Execute\x12@\n\x0e\x45xecuteWindows\x12\x1b.sliverpb.ExecuteWindowsReq\x1a\x11.sliverpb.Execute\x12\x35\n\x08Sideload\x12\x15.sliverpb.SideloadReq\x1a\x12.sliverpb.Sideload\x12;\n\x08SpawnDll\x12\x1b.sliverpb.InvokeSpawnDllReq\x1a\x12.sliverpb.SpawnDll\x12;\n\nScreenshot\x12\x17.sliverpb.ScreenshotReq\x1a\x14.sliverpb.Screenshot\x12P\n\x11\x43urrentTokenOwner\x12\x1e.sliverpb.CurrentTokenOwnerReq\x1a\x1b.sliverpb.CurrentTokenOwner\x12N\n\x12PivotStartListener\x12\x1f.sliverpb.PivotStartListenerReq\x1a\x17.sliverpb.PivotListener\x12\x44\n\x11PivotStopListener\x12\x1e.sliverpb.PivotStopListenerReq\x1a\x0f.commonpb.Empty\x12N\n\x15PivotSessionListeners\x12\x1b.sliverpb.PivotListenersReq\x1a\x18.sliverpb.PivotListeners\x12\x33\n\nPivotGraph\x12\x0f.commonpb.Empty\x1a\x14.clientpb.PivotGraph\x12@\n\x0cStartService\x12\x19.sliverpb.StartServiceReq\x1a\x15.sliverpb.ServiceInfo\x12>\n\x0bStopService\x12\x18.sliverpb.StopServiceReq\x1a\x15.sliverpb.ServiceInfo\x12\x42\n\rRemoveService\x12\x1a.sliverpb.RemoveServiceReq\x1a\x15.sliverpb.ServiceInfo\x12\x38\n\tMakeToken\x12\x16.sliverpb.MakeTokenReq\x1a\x13.sliverpb.MakeToken\x12-\n\x06GetEnv\x12\x10.sliverpb.EnvReq\x1a\x11.sliverpb.EnvInfo\x12/\n\x06SetEnv\x12\x13.sliverpb.SetEnvReq\x1a\x10.sliverpb.SetEnv\x12\x35\n\x08UnsetEnv\x12\x15.sliverpb.UnsetEnvReq\x1a\x12.sliverpb.UnsetEnv\x12\x35\n\x08\x42\x61\x63kdoor\x12\x15.sliverpb.BackdoorReq\x1a\x12.sliverpb.Backdoor\x12\x41\n\x0cRegistryRead\x12\x19.sliverpb.RegistryReadReq\x1a\x16.sliverpb.RegistryRead\x12\x44\n\rRegistryWrite\x12\x1a.sliverpb.RegistryWriteReq\x1a\x17.sliverpb.RegistryWrite\x12P\n\x11RegistryCreateKey\x12\x1e.sliverpb.RegistryCreateKeyReq\x1a\x1b.sliverpb.RegistryCreateKey\x12P\n\x11RegistryDeleteKey\x12\x1e.sliverpb.RegistryDeleteKeyReq\x1a\x1b.sliverpb.RegistryDeleteKey\x12T\n\x13RegistryListSubKeys\x12\x1f.sliverpb.RegistrySubKeyListReq\x1a\x1c.sliverpb.RegistrySubKeyList\x12S\n\x12RegistryListValues\x12\x1f.sliverpb.RegistryListValuesReq\x1a\x1c.sliverpb.RegistryValuesList\x12>\n\rRunSSHCommand\x12\x17.sliverpb.SSHCommandReq\x1a\x14.sliverpb.SSHCommand\x12\x38\n\tHijackDLL\x12\x16.clientpb.DllHijackReq\x1a\x13.clientpb.DllHijack\x12\x35\n\x08GetPrivs\x12\x15.sliverpb.GetPrivsReq\x1a\x12.sliverpb.GetPrivs\x12W\n\x15StartRportFwdListener\x12\".sliverpb.RportFwdStartListenerReq\x1a\x1a.sliverpb.RportFwdListener\x12S\n\x14GetRportFwdListeners\x12\x1e.sliverpb.RportFwdListenersReq\x1a\x1b.sliverpb.RportFwdListeners\x12U\n\x14StopRportFwdListener\x12!.sliverpb.RportFwdStopListenerReq\x1a\x1a.sliverpb.RportFwdListener\x12;\n\x0bOpenSession\x12\x15.sliverpb.OpenSession\x1a\x15.sliverpb.OpenSession\x12\x37\n\x0c\x43loseSession\x12\x16.sliverpb.CloseSession\x1a\x0f.commonpb.Empty\x12P\n\x11RegisterExtension\x12\x1e.sliverpb.RegisterExtensionReq\x1a\x1b.sliverpb.RegisterExtension\x12\x44\n\rCallExtension\x12\x1a.sliverpb.CallExtensionReq\x1a\x17.sliverpb.CallExtension\x12G\n\x0eListExtensions\x12\x1b.sliverpb.ListExtensionsReq\x1a\x18.sliverpb.ListExtensions\x12N\n\x12WGStartPortForward\x12\x1f.sliverpb.WGPortForwardStartReq\x1a\x17.sliverpb.WGPortForward\x12L\n\x11WGStopPortForward\x12\x1e.sliverpb.WGPortForwardStopReq\x1a\x17.sliverpb.WGPortForward\x12<\n\x0cWGStartSocks\x12\x19.sliverpb.WGSocksStartReq\x1a\x11.sliverpb.WGSocks\x12:\n\x0bWGStopSocks\x12\x18.sliverpb.WGSocksStopReq\x1a\x11.sliverpb.WGSocks\x12K\n\x10WGListForwarders\x12\x1c.sliverpb.WGTCPForwardersReq\x1a\x19.sliverpb.WGTCPForwarders\x12K\n\x12WGListSocksServers\x12\x1b.sliverpb.WGSocksServersReq\x1a\x18.sliverpb.WGSocksServers\x12,\n\x05Shell\x12\x12.sliverpb.ShellReq\x1a\x0f.sliverpb.Shell\x12\x32\n\x07Portfwd\x12\x14.sliverpb.PortfwdReq\x1a\x11.sliverpb.Portfwd\x12/\n\x0b\x43reateSocks\x12\x0f.sliverpb.Socks\x1a\x0f.sliverpb.Socks\x12.\n\nCloseSocks\x12\x0f.sliverpb.Socks\x1a\x0f.commonpb.Empty\x12:\n\nSocksProxy\x12\x13.sliverpb.SocksData\x1a\x13.sliverpb.SocksData(\x01\x30\x01\x12\x32\n\x0c\x43reateTunnel\x12\x10.sliverpb.Tunnel\x1a\x10.sliverpb.Tunnel\x12\x30\n\x0b\x43loseTunnel\x12\x10.sliverpb.Tunnel\x1a\x0f.commonpb.Empty\x12<\n\nTunnelData\x12\x14.sliverpb.TunnelData\x1a\x14.sliverpb.TunnelData(\x01\x30\x01\x12,\n\x06\x45vents\x12\x0f.commonpb.Empty\x1a\x0f.clientpb.Event0\x01\x42,Z*github.com/bishopfox/sliver/protobuf/rpcpbb\x06proto3"
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "rpcpb.services_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"Z*github.com/bishopfox/sliver/protobuf/rpcpb"
    _SLIVERRPC._serialized_start = 101
    _SLIVERRPC._serialized_end = 7881
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: commonpb/common.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x15\x63ommonpb/common.proto\x12\x08\x63ommonpb"\x07\n\x05\x45mpty"N\n\x07Request\x12\r\n\x05\x41sync\x18\x01 \x01(\x08\x12\x0f\n\x07Timeout\x18\x02 \x01(\x03\x12\x10\n\x08\x42\x65\x61\x63onID\x18\x08 \x01(\t\x12\x11\n\tSessionID\x18\t \x01(\t"H\n\x08Response\x12\x0b\n\x03\x45rr\x18\x01 \x01(\t\x12\r\n\x05\x41sync\x18\x02 \x01(\x08\x12\x10\n\x08\x42\x65\x61\x63onID\x18\x08 \x01(\t\x12\x0e\n\x06TaskID\x18\t \x01(\t""\n\x04\x46ile\x12\x0c\n\x04Name\x18\x01 \x01(\t\x12\x0c\n\x04\x44\x61ta\x18\x02 \x01(\x0c"\x81\x01\n\x07Process\x12\x0b\n\x03Pid\x18\x01 \x01(\x05\x12\x0c\n\x04Ppid\x18\x02 \x01(\x05\x12\x12\n\nExecutable\x18\x03 \x01(\t\x12\r\n\x05Owner\x18\x04 \x01(\t\x12\x14\n\x0c\x41rchitecture\x18\x07 \x01(\t\x12\x11\n\tSessionID\x18\x05 \x01(\x05\x12\x0f\n\x07\x43mdLine\x18\x06 \x03(\t"$\n\x06\x45nvVar\x12\x0b\n\x03Key\x18\x01 \x01(\t\x12\r\n\x05Value\x18\x02 \x01(\tB/Z-github.com/bishopfox/sliver/protobuf/commonpbb\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "commonpb.common_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"Z-github.com/bishopfox/sliver/protobuf/commonpb"
    _EMPTY._serialized_start = 35
    _EMPTY._serialized_end = 42
    _REQUEST._serialized_start = 44
    _REQUEST._serialized_end = 122
    _RESPONSE._serialized_start = 124
    _RESPONSE._serialized_end = 196
    _FILE._serialized_start = 198
    _FILE._serialized_end = 232
    _PROCESS._serialized_start = 235
    _PROCESS._serialized_end = 364
    _ENVVAR._serialized_start = 366
    _ENVVAR._serialized_end = 402
# @@protoc_insertion_point(module_scope)
